unit uFunctions;

interface

uses Windows, Messages, KOL, SetupAPI;

const
//
// Device registry property codes
// (Codes marked as read-only (R) may only be used for
// SetupDiGetDeviceRegistryProperty)
//
// These values should cover the same set of registry properties
// as defined by the CM_DRP codes in cfgmgr32.h.
//

  SPDRP_DEVICEDESC                  = $00000000; // DeviceDesc (R/W)
  SPDRP_HARDWAREID                  = $00000001; // HardwareID (R/W)
  SPDRP_COMPATIBLEIDS               = $00000002; // CompatibleIDs (R/W)
  SPDRP_UNUSED0                     = $00000003; // unused
  SPDRP_SERVICE                     = $00000004; // Service (R/W)
  SPDRP_UNUSED1                     = $00000005; // unused
  SPDRP_UNUSED2                     = $00000006; // unused
  SPDRP_CLASS                       = $00000007; // Class (R--tied to ClassGUID)
  SPDRP_CLASSGUID                   = $00000008; // ClassGUID (R/W)
  SPDRP_DRIVER                      = $00000009; // Driver (R/W)
  SPDRP_CONFIGFLAGS                 = $0000000A; // ConfigFlags (R/W)
  SPDRP_MFG                         = $0000000B; // Mfg (R/W)
  SPDRP_FRIENDLYNAME                = $0000000C; // FriendlyName (R/W)
  SPDRP_LOCATION_INFORMATION        = $0000000D; // LocationInformation (R/W)
  SPDRP_PHYSICAL_DEVICE_OBJECT_NAME = $0000000E; // PhysicalDeviceObjectName (R)
  SPDRP_CAPABILITIES                = $0000000F; // Capabilities (R)
  SPDRP_UI_NUMBER                   = $00000010; // UiNumber (R)
  SPDRP_UPPERFILTERS                = $00000011; // UpperFilters (R/W)
  SPDRP_LOWERFILTERS                = $00000012; // LowerFilters (R/W)
  SPDRP_BUSTYPEGUID                 = $00000013; // BusTypeGUID (R)
  SPDRP_LEGACYBUSTYPE               = $00000014; // LegacyBusType (R)
  SPDRP_BUSNUMBER                   = $00000015; // BusNumber (R)
  SPDRP_ENUMERATOR_NAME             = $00000016; // Enumerator Name (R)
  SPDRP_SECURITY                    = $00000017; // Security (R/W, binary form)
  SPDRP_SECURITY_SDS                = $00000018; // Security (W, SDS form)
  SPDRP_DEVTYPE                     = $00000019; // Device Type (R/W)
  SPDRP_EXCLUSIVE                   = $0000001A; // Device is exclusive-access (R/W)
  SPDRP_CHARACTERISTICS             = $0000001B; // Device Characteristics (R/W)
  SPDRP_ADDRESS                     = $0000001C; // Device Address (R)
  SPDRP_UI_NUMBER_DESC_FORMAT       = $0000001D; // UiNumberDescFormat (R/W)
  SPDRP_DEVICE_POWER_DATA           = $0000001E;
  SPDRP_REMOVAL_POLICY              = $0000001F;
  SPDRP_REMOVAL_POLICY_HW_DEFAULT   = $00000020;
  SPDRP_REMOVAL_POLICY_OVERRIDE     = $00000021;
  SDRP_INSTALL_STATE                = $00000022;
  // 
  SPDRP_DEVICE_NAME                 = $00000023;
  SPDRP_NULL                        = $00000099;

var
  DeviceGUIDList: PStrListEx;
  DeviceInfoData: SP_DEVINFO_DATA;
  DeviceListHandle: HDEVINFO;

implementation

uses uInterface;

function GUIDToString(G: TGUID): string;
var
  i: Integer;
begin
  Result := '{' + Int2Hex(G.D1, 8) + '-' + Int2Hex(G.D2, 4) + '-' + Int2Hex(G.D3, 4) + '-';
  for i := 0 to 7 do
  begin
    Result := Result + Int2Hex(G.D4[i], 2);
    if i = 1 then Result := Result + '-'
  end;
  Result := Result + '}';
end;

function StringToGUID(s: string): TGUID;
var
  i: Integer;
  s1, s2, s3, s4: string;
begin
  {4D36E965-E325-11CE-BFC1-08002BE10318}
  s1 := Copy(s, 2, 8);
  s2 := Copy(s, 11, 4);
  s3 := Copy(s, 16, 4);
  s4 := Copy(s, 21, 2) + Copy(s, 23, 2) + Copy(s, 26, 12);

  Result.D1 := Hex2Int(s1);
  Result.D2 := Hex2Int(s2);
  Result.D3 := Hex2Int(s3);     

  for i := 0 to 7 do
  begin
    Result.D4[i] := Hex2Int(Copy(s4, 1, 2));
    Delete(s4, 1, 2);
  end;
end;

function ExtractMultiString(const Value: String): String;
var
  P: PChar;
begin
  P := @Value[1];
  while P^ <> #0 do
  begin
    if Result <> '' then
      Result := Result + ', ';
    Result := Result + P;
    Inc(P, lstrlen(P) + 1);
  end;
end;

function GetString(PropertyCode: Integer): String;
var
  dwPropertyRegDataType, dwRequiredSize: DWORD;
begin             
  Result := '';
  dwRequiredSize := 0;
  dwPropertyRegDataType := REG_SZ;
  SetupDiGetDeviceRegistryPropertyA(DeviceListHandle, DeviceInfoData,
    PropertyCode, dwPropertyRegDataType, nil, 0, dwRequiredSize);
  if not (dwPropertyRegDataType in [REG_SZ, REG_MULTI_SZ]) then Exit;
  if GetLastError = ERROR_INSUFFICIENT_BUFFER then
  begin
    SetLength(Result, dwRequiredSize);
    SetupDiGetDeviceRegistryPropertyA(DeviceListHandle, DeviceInfoData,
      PropertyCode, dwPropertyRegDataType, @Result[1],
      dwRequiredSize, dwRequiredSize);
  end;
  case dwPropertyRegDataType of
    REG_SZ: Result := PChar(Result);
    REG_MULTI_SZ: Result := ExtractMultiString(Result);
  end;
end;

function GetGuid(PropertyCode: Integer): TGUID;
var
  dwPropertyRegDataType, dwRequiredSize: DWORD;
  StringGUID: String;
begin
  ZeroMemory(@Result, SizeOf(TGUID));
  StringGUID := GetString(PropertyCode);
  if StringGUID = '' then
  begin
    dwRequiredSize := 0;
    dwPropertyRegDataType := REG_BINARY;
    SetupDiGetDeviceRegistryPropertyA(DeviceListHandle, DeviceInfoData,
      PropertyCode, dwPropertyRegDataType, nil, 0, dwRequiredSize);
    if GetLastError = ERROR_INSUFFICIENT_BUFFER then
    begin
      SetupDiGetDeviceRegistryPropertyA(DeviceListHandle, DeviceInfoData,
        PropertyCode, dwPropertyRegDataType, @Result,
        dwRequiredSize, dwRequiredSize);
    end;
  end
  else
    Result := StringToGUID(StringGUID);
end;

function DeviceClassDescription(DeviceTypeGUID: TGUID): String;
var
  dwRequiredSize: DWORD;
begin
  Result := '';
  dwRequiredSize := 0;
  SetupDiGetClassDescriptionA(DeviceTypeGUID, nil, 0, dwRequiredSize);
  if GetLastError = ERROR_INSUFFICIENT_BUFFER then
  begin
    SetLength(Result, dwRequiredSize);
    SetupDiGetClassDescriptionA(DeviceTypeGUID, @Result[1], dwRequiredSize, dwRequiredSize);
  end;
  Result := PChar(Result);
end;

{function AddDevices(aGUID: TGUID; dvProp: DWORD): string;
var
  PnPHandle: HDEVINFO;
  DevData: TSPDevInfoData;
  RES: LongBool;
  Devn: Integer;
  _DN: ULONG;
begin
  Result := '';
  PnPHandle := SetupDiGetClassDevs(@aGUID, nil, 0, DIGCF_PRESENT);
  if PnPHandle = INVALID_HANDLE_VALUE then Exit;

  Devn := 0;
  repeat
    DevData.cbSize := SizeOf(DevData);
    RES := SetupDiEnumDeviceInfo(PnPHandle, Devn, DevData);
    if (RES) and (_DN <> DN_ROOT_ENUMERATED) then
    begin
      case dvProp of
        SPDRP_DEVICE_NAME: Result := GetDeviceName(PnPHandle, DevData);
      else
        Result := GetString(dvProp, PnPHandle, DevData);
      end;


      Inc(Devn);
    end;
    if Devn = 0 then
    begin
      break;
    end;
  until not RES;
  SetupDiDestroyDeviceInfoList(PnPHandle);
end;
            }
procedure EnumAllDevices(ClassGUID: TGUID);
const
  PINVALID_HANDLE_VALUE = Pointer(INVALID_HANDLE_VALUE);
var
  dwFlags: DWORD;
  dwIndex: DWORD;

  idx: Integer;
  
  DeviceName, DeviceClassName: String;
  DeviceClassesCount, DevicesCount: Integer;
begin
  //DVD  CD-ROM    {4D36E965-E325-11CE-BFC1-08002BE10318}
  //       {4D36E967-E325-11CE-BFC1-08002BE10318}
  DeviceGUIDList.Clear;

  //      
  dwFlags := DIGCF_ALLCLASSES;// or DIGCF_DEVICEINTERFACE;
//  dwFlags := dwFlags {or DIGCF_PRESENT}; //   

  //    DIS (Device Information Sets)
  //     
  DeviceListHandle := SetupDiGetClassDevsExA(@ClassGUID, nil, 0, dwFlags, 0, nil, nil);
  if DeviceListHandle = INVALID_HANDLE_VALUE then Exit;

  {============================================================================}
  try
    dwIndex := 0;
    DeviceClassesCount := 0;
    DevicesCount := 0;

    //     
    ZeroMemory(@DeviceInfoData, SizeOf(SP_DEVINFO_DATA));
    DeviceInfoData.cbSize := SizeOf(SP_DEVINFO_DATA);

    //       DIS
    //     dwIndex
    while SetupDiEnumDeviceInfo(DeviceListHandle, dwIndex, DeviceInfoData) do
    begin
      //     (FriendlyName)
      DeviceName := GetString(SPDRP_FRIENDLYNAME);

      DeviceGUIDList.AddObject(DeviceName, );

      //     -     
      if DeviceName = '' then DeviceName := GetString(SPDRP_DEVICEDESC);

      //  GUID ,    
      ClassGUID := GetGuid(SPDRP_CLASSGUID);
      //   ,    
      DeviceClassName := DeviceClassDescription(ClassGUID);

      //DeviceClassesList.Add(DeviceClassName);
      Inc(DeviceClassesCount);
      Inc(DevicesCount);

      //    
      Inc(dwIndex);
    end;

 {   StatusBar.Panels[0].Text := 'DeviceClasses Count: ' +
      IntToStr(DeviceClassesCount);
    StatusBar.Panels[1].Text := 'Devices Count: ' +
      IntToStr(DevicesCount);   }
    
  finally end;
  Text2Clipboard(DeviceGUIDList.Text);
end;

procedure TestFunction; cdecl;
begin
  //MsgOK('This is the MsgOK');
  //SendScriptCommand('Message("","")', False);
  //CallMMB(True, True);
end;

procedure EnumDevices; cdecl;
var
  DevClass: TGUID;
begin
  case NumInput[1] of
    0: ;
    //Bluetooth
    1: DevClass := StringToGUID('{7240100F-6512-4548-8418-9EBB5C6A1A94}');
    //DVD  CD-ROM 
    2: DevClass := StringToGUID('{4D36E965-E325-11CE-BFC1-08002BE10318}');
    //IDE ATA/ATAPI 
    3: DevClass := StringToGUID('{4D36E96A-E325-11CE-BFC1-08002BE10318}');
    //Personal identification devices
    4: DevClass := StringToGUID('{53D29EF7-377C-4D14-864B-EB3A85769359}');
    //SCSI  RAID 
    5: DevClass := StringToGUID('{4D36E97B-E325-11CE-BFC1-08002BE10318}');
    //Secure Digital -
    6: DevClass := StringToGUID('{A0A588A4-C46F-4B37-B7EA-C82FE89870C6}');
    //
    7: DevClass := StringToGUID('{72631E54-78A4-11D0-BCF7-00AA00B7B32A}');
    //
    8: DevClass := StringToGUID('{4D36E968-E325-11CE-BFC1-08002BE10318}');
    // 
    9: DevClass := StringToGUID('{4D36E967-E325-11CE-BFC1-08002BE10318}');
    //   Plug and Play
    10: DevClass := StringToGUID('{8ECC055D-047F-11D1-A537-0000F8753ED1}');
    //,    
    11: DevClass := StringToGUID('{4D36E96C-E325-11CE-BFC1-08002BE10318}');
    //
    12: DevClass := StringToGUID('{4D36E96B-E325-11CE-BFC1-08002BE10318}');
    //  61883
    13: DevClass := StringToGUID('{7EBEFBC0-3200-11D2-B4C2-00A0C9697D07}');
    //  AVC
    14: DevClass := StringToGUID('{C06FF265-AE09-48F0-812C-16753D7CBA83}');
    //
    15: DevClass := StringToGUID('{4D36E966-E325-11CE-BFC1-08002BE10318}');
    //    USB
    16: DevClass := StringToGUID('{36FC9E60-C465-11CF-8056-444553540000}');
    //
    17: DevClass := StringToGUID('{4D36E96D-E325-11CE-BFC1-08002BE10318}');
    //
    18: DevClass := StringToGUID('{4D36E96E-E325-11CE-BFC1-08002BE10318}');
    //    
    19: DevClass := StringToGUID('{4D36E96F-E325-11CE-BFC1-08002BE10318}');
    // (COM  LPT)
    20: DevClass := StringToGUID('{4D36E978-E325-11CE-BFC1-08002BE10318}');
    //
    21: DevClass := StringToGUID('{50127DC3-0F36-415E-A6CC-4CB3BE910B65}');
    // 
    22: DevClass := StringToGUID('{4D36E972-E325-11CE-BFC1-08002BE10318}');
    // 
    23: DevClass := StringToGUID('{4D36E97D-E325-11CE-BFC1-08002BE10318}');
    //  
    24: DevClass := StringToGUID('{71A27CDD-812A-11D0-BEC7-08002BE2092F}');
    // HID (Human Interface Devices)
    25: DevClass := StringToGUID('{745A17A0-74D3-11D0-B6FE-00A0C90F57DA}');
    //  
    26: DevClass := StringToGUID('{6BDD1FC6-810F-11D0-BEC7-08002BE2092F}');
    //-  IEEE 1394
    27: DevClass := StringToGUID('{6BDD1FC1-810F-11D0-BEC7-08002BE2092F}');
  end;  

  EnumAllDevices(DevClass);
end;

procedure GetProperty; cdecl;
var
  dvProp: DWORD;
  _i: DWORD;
  GUID: PGUID;
  s: string;
begin
  TextOutput := '';
  dvProp := SPDRP_NULL;
  
  if TextInput[1] = 'DeviceName' then dvProp := SPDRP_DEVICE_NAME;
  if TextInput[1] = 'DeviceDesc' then dvProp := SPDRP_DEVICEDESC;
  if TextInput[1] = 'HardwareID' then dvProp := SPDRP_HARDWAREID;
  if TextInput[1] = 'Class_Name' then dvProp := SPDRP_CLASS;

  if dvProp = SPDRP_NULL then Exit;

{  for _i := 0 to DeviceClassesList.Count - 1 do
  begin
    GUID := PGUID(DeviceClassesList.Objects[_i]);
    s := AddDevices(GUID^, dvProp);
    TextOutput := TextOutput + s + '#';
  end;
  NumOutput := DeviceClassesList.Count;  }
end;

exports
  EnumDevices,
  GetProperty

  ;
      
initialization
  DeviceGUIDList := NewStrListEx;

finalization
  DeviceGUIDList.Free;

end.

